==> unixODBC-1.8.8.tar.gz: unixODBC-1.8.8/sqi/_sqiOpenTable.c <==
#include "sqi.h"

/********************************************
 * _sqiLoadTable
 *
 * Loads a root table from a table file. I used
 * to load these into a global table cache but ran
 * into some probs and started running out of time
 * so many compromises were made <sigh>. I have left
 * much of the strucure in place to do so in the
 * future. - PAH
 *
 ********************************************/

int _sqiLoadTableData( HSQITABLE hTable, FILE *hFile );
int _sqiStoreColumnValue( HSQITABLE hTable, int nRow, int nCol, char **aCols, char *pszValue );
int _sqiStoreRow( HSQITABLE hTable, int nRow, int nCol, char **aCols );


int _sqiOpenTable( HSQITABLE *phTable, char *pszCatalogFile, char *pszTableName, char *pszTableFile  )
{
	HSQITABLE	hTable;
    FILE    	*hFile;

	if ( !phTable )
		return false;
			
	if ( !pszCatalogFile )
		return false;

	hFile = fopen( pszTableFile, "r" );
	if ( !hFile )
		hFile = fopen( pszTableFile, "w" );

	if ( !hFile )
		return false;

	/******************************
	 * INIT NEW HSQITABLE
	 ******************************/
	hTable = malloc( sizeof(SQITABLE) );
	hTable->pszTableName		= (char*)strdup(pszTableName);
	hTable->pszFileName 		= (char*)strdup(pszTableFile);
	hTable->hColHdrs			= lstOpen();
	hTable->hRows				= lstOpen();
	hTable->nRefs				= 1;				/* init to 1 if you want to keep the table loaded even when no refs... ie for a table cache */
	hTable->hBase				= NULL;

	/******************************
	 * USE CUSTOM FREE FUNCS TO ENSURE ALL MEM IS FREED
	 ******************************/
	lstSetFreeFunc( hTable->hColHdrs, _sqiFreeColumnHeader );
	lstSetFreeFunc( hTable->hRows, _sqiFreeDataRow );

	/******************************
	 * LOAD FILE
	 ******************************/
	_sqiLoadTableColHdrs( pszCatalogFile, hTable->pszTableName, hTable->hColHdrs, hFile );
	if ( hTable->hColHdrs->nItems > 0 )
		_sqiLoadTableData( hTable, hFile );

	fclose( hFile );

	*phTable = hTable;

	return true;
}


/**********************
 * _sqiLoadTableData
 *
 * Each row item has a pData which is a pointer to an array of pointers.
 *
 * So this...
 *
 * hRows->hCurrent->hItem->pData[nCol]
 *
 * is a pointer to a string which contains the col value.
 *
 * For example:  printf( hRows->hCurrent->hItem->pData[nCol] );
 *
 **********************/
int _sqiLoadTableData( HSQITABLE hTable, FILE *hFile )
{	
	char	**aCols;
	int		nCol;									/* zero based */
	int		nRow;
	int		nChar;
	char	szColumnValue[300]	= "";
	char	cChar;
	char	cDelimiter			= ',';

	/*
    memset( aCols, '\0', sizeof(aCols) );	
	*/

    aCols = malloc( sizeof( char * ) * hTable->hColHdrs->nItems+1 );

    aCols[hTable->hColHdrs->nItems] = NULL;

	nCol = 0;
	nRow = 1;
	nChar= 0;
	while ( (cChar = fgetc( hFile )) != EOF )
	{
		if ( cChar == cDelimiter )
		{
			/**********************************
			 * STORE COLUMN VALUE
			 **********************************/
			szColumnValue[nChar] = '\0';
			nChar = 0;
			_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
			nCol++;
		}
		else if ( cChar == '\n' )
		{
			if ( nCol > 0 || nChar > 0 ) /* skip lines only containing newline char */
			{
				/**********************************
				 * STORE COLUMN VALUE
				 **********************************/
				szColumnValue[nChar] = '\0';
				nChar = 0;
				_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
				nCol++;

				/**********************************
				 * STORE COLUMN VALUE
				 **********************************/
				_sqiStoreRow( hTable, nRow, nCol, aCols );
				nRow++;
				nCol = 0;
			}
		}
		else
		{
			szColumnValue[nChar] = cChar;
			nChar++;
		}
	}

	if ( nCol > 0 || nChar > 0 )
	{
		/**********************************
		 * STORE COLUMN VALUE
		 **********************************/
		szColumnValue[nChar] = '\0';
		nChar = 0;
		_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
		nCol++;

		/**********************************
		 * STORE COLUMN VALUE
		 **********************************/
		_sqiStoreRow( hTable, nRow, nCol, aCols );
		nRow++;
		nCol = 0;
	}

    free( aCols );

	return true;
}


int _sqiStoreColumnValue( HSQITABLE hTable, int nRow, int nCol, char **aCols, char *pszValue )
{
	if ( nCol + 1 > hTable->hColHdrs->nItems )
		printf( "[SQI][%s][%d] Too many columns on row (%d)", __FILE__, __LINE__, nRow );
	else
		aCols[nCol] = (char*)strdup( pszValue );

	return true;
}


int _sqiStoreRow( HSQITABLE hTable, int nRow, int nCol, char **aCols )
{
	char	*pData;

	/*****************
	 * TOO FEW COLS?
	 *****************/
	if ( nCol+1 < hTable->hColHdrs->nItems )
	{
		printf( "[SQI][%s][%d] Too few columns on row (%d)", __FILE__, __LINE__, nRow );

		while ( nCol + 1 < hTable->hColHdrs->nItems )
		{
			aCols[nCol] = (char*)strdup( "" );
			nCol++;
		}
	}

	/*****************
	 * DUP & STORE
	 *****************/
	pData = malloc( sizeof(char*) * hTable->hColHdrs->nItems );
	memcpy( pData, aCols, sizeof(char*) * hTable->hColHdrs->nItems );
	lstAppend( hTable->hRows, pData );

	return true;
}

==> unixODBC-1.8.13.tar.gz: unixODBC-1.8.13/sqi/_sqiOpenTable.c <==
#include "sqi.h"

/********************************************
 * 
 *
 * Loads a root table from a table file. I used
 * to load these into a global table cache but ran
 * into some probs and started running out of time
 * so many compromises were made <sigh>. I have left
 * much of the strucure in place to do so in the
 * future. - PAH
 *
 ********************************************/

int _sqiStoreColumnValue( HSQITABLE hTable, int nRow, int nCol, char **aCols, char *pszValue );
int _sqiStoreRow( HSQITABLE hTable, int nRow, int nCol, char **aCols );


/********************************************
 * 
 * OBSOLETE
 *
 * _sqiOpenTable
 *
 * This takes column headers from the top of the
 * table data file. It may be used in the future
 * but it is not called right now.
 *
 ********************************************/
int _sqiOpenTable( HSQITABLE *phTable, char *pszCatalogFile, char *pszTableName, char *pszTableFile  )
{
	HSQITABLE	hTable;
    FILE    	*hFile;

	if ( !phTable )
		return false;
			
	if ( !pszCatalogFile )
		return false;

	hFile = fopen( pszTableFile, "r" );
	if ( !hFile )
		hFile = fopen( pszTableFile, "w" );

	if ( !hFile )
		return false;

	/******************************
	 * INIT NEW HSQITABLE
	 ******************************/
	hTable = malloc( sizeof(SQITABLE) );
	hTable->pszTableName		= (char*)strdup(pszTableName);
	hTable->pszFileName 		= (char*)strdup(pszTableFile);
	hTable->hColHdrs			= lstOpen();
	hTable->hRows				= lstOpen();
	hTable->nRefs				= 1;				/* init to 1 if you want to keep the table loaded even when no refs... ie for a table cache */
	hTable->hBase				= NULL;

	/******************************
	 * USE CUSTOM FREE FUNCS TO ENSURE ALL MEM IS FREED
	 ******************************/
	lstSetFreeFunc( hTable->hColHdrs, _sqiFreeColumnHeader );
	lstSetFreeFunc( hTable->hRows, _sqiFreeDataRow );

	/******************************
	 * LOAD FILE
	 ******************************/
	_sqiLoadTableColHdrs( pszCatalogFile, hTable->pszTableName, hTable->hColHdrs, hFile );
	if ( hTable->hColHdrs->nItems > 0 )
		_sqiLoadTableData( hTable, hFile );

	fclose( hFile );

	*phTable = hTable;

	return true;
}


/**********************
 * _sqiLoadTableData
 *
 * Each row item has a pData which is a pointer to an array of pointers.
 *
 * So this...
 *
 * hRows->hCurrent->hItem->pData[nCol]
 *
 * is a pointer to a string which contains the col value.
 *
 * For example:  printf( hRows->hCurrent->hItem->pData[nCol] );
 *
 **********************/
int _sqiLoadTableData( HSQITABLE hTable, FILE *hFile )
{	
	char	**aCols;
	int		nCol;									/* zero based */
	int		nRow;
	int		nChar;
	char	szColumnValue[300]	= "";
	char	cChar;
	char	cDelimiter			= ',';

    aCols = malloc( sizeof(char*) * hTable->hColHdrs->nItems + 2 );
    memset( aCols, 0, sizeof(aCols) + 2 );	

	lstFirst( hTable->hColHdrs );
	nCol = 0;
	nRow = 1;
	nChar= 0;
	while ( (cChar = fgetc( hFile )) != EOF )
	{
		if ( cChar == cDelimiter )
		{
			/**********************************
			 * STORE COLUMN VALUE
			 **********************************/
			szColumnValue[nChar] = '\0';
			nChar = 0;
			_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
			nCol++;

			lstNext( hTable->hColHdrs );
		}
		else if ( cChar == '\n' )
		{
			if ( nCol > 0 || nChar > 0 ) /* skip lines only containing newline char */
			{
				/**********************************
				 * STORE COLUMN VALUE
				 **********************************/
				szColumnValue[nChar] = '\0';
				nChar = 0;
				_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
				nCol++;

				/**********************************
				 * STORE ROW
				 **********************************/
				_sqiStoreRow( hTable, nRow, nCol, aCols );
				nRow++;
				nCol = 0;

				lstFirst( hTable->hColHdrs );
			}
		}
		else
		{
			szColumnValue[nChar] = cChar;
			nChar++;
		}
	}

	if ( nCol > 0 || nChar > 0 )
	{
		/**********************************
		 * STORE COLUMN VALUE
		 **********************************/
		szColumnValue[nChar] = '\0';
		nChar = 0;
		_sqiStoreColumnValue( hTable, nRow, nCol, aCols, szColumnValue );
		nCol++;

		/**********************************
		 * STORE ROW
		 **********************************/
		_sqiStoreRow( hTable, nRow, nCol, aCols );
		nRow++;
		nCol = 0;
	}

    free( aCols );

	return true;
}


int _sqiStoreColumnValue( HSQITABLE hTable, int nRow, int nCol, char **aCols, char *pszValue )
{
	if ( lstEOL( hTable->hColHdrs ) )
	{
		printf( "[SQI][%s][%d] Too many columns on row (%d)", __FILE__, __LINE__, nRow );
		return 0;
	}
	
	aCols[nCol] = (char*)strdup( pszValue );

	return true;
}


int _sqiStoreRow( HSQITABLE hTable, int nRow, int nCol, char **aCols )
{
	char	*pData;

	/*****************
	 * TOO FEW COLS?
	 *****************/
	if ( nCol+1 < hTable->hColHdrs->nItems )
	{
		printf( "[SQI][%s][%d] Too few columns on row (%d)", __FILE__, __LINE__, nRow );

		while ( nCol + 1 < hTable->hColHdrs->nItems )
		{
			aCols[nCol] = (char*)strdup( "" );
			nCol++;
		}
	}

	/*****************
	 * DUP (pointers) & STORE
	 *****************/
	pData = malloc( sizeof(char*) * hTable->hColHdrs->nItems + 1 );
	memcpy( pData, aCols, sizeof(char*) * hTable->hColHdrs->nItems + 1 );
	lstAppend( hTable->hRows, pData );

	return true;
}
